"""
Function wrappers around jinja templates.

This allows for type checking.
"""

from typing import (
    Any,
    Optional,
)
from jinja2 import (
    Environment,
    FileSystemLoader,
)
from .breadcrumbs import Breadcrumbs
from .gather import ModuleEntry

jinja = Environment(
    loader=FileSystemLoader('templates'),
    autoescape=False,
)


def code_page(*,
              title: str,
              content: str,
              path_base: str,
              breadcrumbs: Optional[Breadcrumbs] = None
              ) -> str:  # pragma: no cover
    """Template for a page containing puppet code."""
    template = jinja.get_template('code_page.html')
    return template.render(
            title=title,
            content=content,
            path_base=path_base,
            breadcrumbs=breadcrumbs)


def content(*,
            content: str,
            path_base: str,
            breadcrumbs: Optional[Breadcrumbs] = None
            ) -> str:  # pragma: no cover
    """Template for a page with arbitrary content."""
    template = jinja.get_template('content.html')
    return template.render(
            content=content,
            path_base=path_base,
            breadcrumbs=breadcrumbs)


def index(*,
          modules: list[ModuleEntry],
          path_base: str,
          breadcrumbs: Optional[Breadcrumbs] = None
          ) -> str:  # pragma: no cover
    """Root index file."""
    template = jinja.get_template('index.html')
    return template.render(
            path_base=path_base,
            modules=modules,
            breadcrumbs=breadcrumbs)


def module_index(
        *,
        # content: list[],  # something with to_html_list and to_html
        content: list[Any],  # TODO something with to_html_list and to_html
        module_author: str,
        module_name: str,
        doc_files: list[tuple[str, str]],
        path_base: str,
        breadcrumbs: Optional[Breadcrumbs] = None,
        ) -> str:  # pragma: no cover
    """Index for a single module."""
    template = jinja.get_template('module_index.html')
    return template.render(
            content=content,
            module_author=module_author,
            module_name=module_name,
            doc_files=doc_files,
            path_base=path_base,
            breadcrumbs=breadcrumbs)
